/* Wireless extensions support.
 *
 * See copyright notice in main.c
 */
#ifndef _ORINOCO_WEXT_H_
#define _ORINOCO_WEXT_H_

#include <net/iw_handler.h>

/* Structure defining all our WEXT handlers */
extern const struct iw_handler_def orinoco_handler_def;

#endif /* _ORINOCO_WEXT_H_ */
